package agentkapp

import (
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestParseHeaders(t *testing.T) {
	input := []string{
		"x-custom-header-1:value1",
		"x-custom-header-2: value2",
		"x-custom-header-2 : value3",
		" x-custom-header-3: value 4 ",
	}
	expected := http.Header{
		"X-Custom-Header-1": []string{"value1"},
		"X-Custom-Header-2": []string{"value2", "value3"},
		"X-Custom-Header-3": []string{"value 4"},
	}
	h, err := parseHeaders(input)
	require.NoError(t, err)
	assert.Equal(t, expected, h)
}

func TestCollectExtraTelemetryData(t *testing.T) {
	// GIVEN
	environ := []string{
		"UNRELATED=foo",
		"GITLAB_UNRELATED=bar",
		"INVALID",
		"GITLAB_AGENT_TELEMETRY_KEY=some-value",
		"GITLAB_AGENT_TELEMETRY_ANOTHER_KEY=",
	}

	// WHEN
	data := make(map[string]string, 2)
	collectExtraTelemetryData(environ, data)

	// THEN
	assert.Equal(t, map[string]string{
		"KEY":         "some-value",
		"ANOTHER_KEY": "",
	}, data)
}
