package watch_graph //nolint:stylecheck

import (
	"encoding/json"

	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/utils/ptr"
)

func (vx *wgObject2vertex) collectOwnerReferencesArcs(obj *unstructured.Unstructured) *Error {
	return collectEachErr(obj.GetOwnerReferences(), ownerReferenceArcType, vx.ownerReference2arc)
}

func (vx *wgObject2vertex) ownerReference2arc(ref *metav1.OwnerReference, arcType ArcType) *Error {
	gv, err := schema.ParseGroupVersion(ref.APIVersion)
	if err != nil {
		vx.appendObjectProcessingWarning(err.Error())
		return nil
	}
	var data map[string]any
	data = maybeSet(ptr.Deref(ref.Controller, false), data, "c", true)
	data = maybeSet(ptr.Deref(ref.BlockOwnerDeletion, false), data, "b", true)

	var arcdata ArcData
	if len(data) > 0 {
		arcdata, err = json.Marshal(data)
		if err != nil {
			return &Error{
				Message: err.Error(),
				Code:    InternalError,
			}
		}
	}
	vx.setArcTo(gv.Group, ref.Kind, ref.Name, "", arcdata, arcType)

	return nil
}

func maybeSet(cond bool, mp map[string]any, key string, val bool) map[string]any {
	if !cond {
		return mp
	}
	if mp == nil {
		mp = make(map[string]any)
	}
	mp[key] = val
	return mp
}
