package watch_graph //nolint:stylecheck

import (
	corev1 "k8s.io/api/core/v1"
)

func (vx *wgObject2vertex) collectPodSpecV1Arcs(s *corev1.PodSpec, arcType ArcType) {
	collectEach(s.Volumes, arcType, vx.collectVolumeV1Arcs)
	collectEach(s.InitContainers, arcType, vx.collectContainerV1Arcs)
	collectEach(s.Containers, arcType, vx.collectContainerV1Arcs)
	collectEach(s.EphemeralContainers, arcType, vx.collectEphemeralContainerV1Arcs)

	if s.ServiceAccountName != "" {
		vx.setArcToGVK(serviceAccountGVK, s.ServiceAccountName, nil, referenceArcType)
	}
	if s.NodeName != "" {
		vx.setArcToGVK(nodeGVK, s.NodeName, nil, referenceArcType)
	}

	collectEach(s.ImagePullSecrets, referenceArcType, func(r *corev1.LocalObjectReference, arcType ArcType) {
		vx.setArcToGVK(secretGVK, r.Name, nil, arcType)
	})

	if s.PriorityClassName != "" {
		vx.setArcToGVK(priorityClassGVK, s.PriorityClassName, nil, arcType)
	}

	if s.RuntimeClassName != nil && *s.RuntimeClassName != "" {
		vx.setArcToGVK(runtimeClassGVK, *s.RuntimeClassName, nil, arcType)
	}

	collectEach(s.ResourceClaims, referenceArcType, func(c *corev1.PodResourceClaim, arcType ArcType) {
		switch {
		case c.ResourceClaimName != nil:
			vx.setArcToGVK(resourceClaimV1Beta1GVK, *c.ResourceClaimName, nil, arcType)
		case c.ResourceClaimTemplateName != nil:
			vx.setArcToGVK(resourceClaimTemplateV1Beta1GVK, *c.ResourceClaimName, nil, arcType)
		}
	})
}
