package watch_graph //nolint:stylecheck
import (
	"k8s.io/apimachinery/pkg/runtime/schema"
)

const (
	infSyncFailedWarnType          = "INFORMER_SYNC_FAILED"
	discoFailedWarnType            = "DISCOVERY_FAILED"
	namespaceListFailedWarnType    = "NAMESPACE_LIST_FAILED"
	objectProcessingFailedWarnType = "OBJECT_PROCESSING_FAILED"
)

type Warning struct {
	Type       string
	Message    string
	Attributes map[string]any
}

func NewObjectProcessingWarning(gvr schema.GroupVersionResource, namespace, name, message string) Warning {
	attrs := map[string]any{
		"g": gvr.Group,
		"v": gvr.Version,
		"r": gvr.Resource,
		"n": name,
	}
	if namespace != "" {
		attrs["ns"] = namespace
	}
	return Warning{
		Type:       objectProcessingFailedWarnType,
		Message:    message,
		Attributes: attrs,
	}
}
